package com.zhengyiyu.common;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * @author yiyu
 * Offers the list of Human 24(1-22, X, Y) chromosomes list
 * maybe in the future we need to add more species
 * added Yeast(Saccharomyces cerevisiase) at Nov 22, 2011
 */

public class ChromosomeList {
	
	public static final String CHR_PREFIX = "chr";

	/**
	 * All the normal chr in human, from Chr1-22, ChrX, Chry
	 */
	public static ArrayList<String> HumanChrNameList = new ArrayList<String>();

	/**
	 * All the numerical chr in human, chr1-22, sometimes it is not easy to handle the X, Y to integer, it is for some occasion that we need the chr num to be integer
	 */
	public static ArrayList<String> HumanChrNameList_NoXYM = new ArrayList<String>();

	/**
	 * All the chr in human include the ChrM, nowhere use it yet
	 */
	public static ArrayList<String> HumanChrNameList_WithM = new ArrayList<String>();

	/**
	 * All the chr in yeast (sc), chr1-16, integeral display
	 */
	public static ArrayList<String> YeastChrNameList = new ArrayList<String>();

	/**
	 * All the chr in yeast (sc), chrI - XVI, roman express
	 */
	public static ArrayList<String> YeastChrNameList_Roman = new ArrayList<String>();

	public static HashMap<String, Integer> Hg18ChrLengthMap = new HashMap<String, Integer>();
	
	// to initial all the values in the Chromosome list, chr1-22
	static {
		// for human
		for (int i = 1; i < 23; i++) {
			HumanChrNameList.add(CHR_PREFIX + i);
			HumanChrNameList_NoXYM.add(CHR_PREFIX + i);
			HumanChrNameList_WithM.add(CHR_PREFIX + i);
		}

		// normally human chr have chrX and chrY
		HumanChrNameList.add(CHR_PREFIX + "X");
		HumanChrNameList.add(CHR_PREFIX + "Y");

		// Actually we do not need chrM at least now, but maybe someday later
		HumanChrNameList_WithM.add(CHR_PREFIX + "X");
		HumanChrNameList_WithM.add(CHR_PREFIX + "Y");
		HumanChrNameList_WithM.add(CHR_PREFIX + "M");

		// for yeast
		// yeast only have 16 chromosomes.
		for (int i = 1; i <= 16; i++) {
			YeastChrNameList.add(CHR_PREFIX + i);
		}

		// in some gene format (now in ucsc, yeast uses roman number to order
		// chr)
		String[] romanYeastChrList = { "I", "II", "III", "IV", "V", "VI",
				"VII", "VIII", "IX", "X", "XI", "XII", "XIII", "XIV", "XV",
				"XVI" };

		for (String chr : romanYeastChrList) {
			YeastChrNameList_Roman.add(CHR_PREFIX + chr);
		}
		
		Hg18ChrLengthMap.put("chr1", 247249719);
		Hg18ChrLengthMap.put("chr2", 242951149);
		Hg18ChrLengthMap.put("chr3", 199501827);
		Hg18ChrLengthMap.put("chr4", 191273063);
		Hg18ChrLengthMap.put("chr5", 180857866);
		Hg18ChrLengthMap.put("chr6", 170899992);
		Hg18ChrLengthMap.put("chr7", 158821424);
		Hg18ChrLengthMap.put("chr8", 146274826);
		Hg18ChrLengthMap.put("chr9", 140273252);
		Hg18ChrLengthMap.put("chr10", 135374737);
		Hg18ChrLengthMap.put("chr11", 134452384);
		Hg18ChrLengthMap.put("chr12", 132349534);
		Hg18ChrLengthMap.put("chr13", 114142980);
		Hg18ChrLengthMap.put("chr14", 106368585);
		Hg18ChrLengthMap.put("chr15", 100338915);
		Hg18ChrLengthMap.put("chr16", 88827254);
		Hg18ChrLengthMap.put("chr17", 78774742);
		Hg18ChrLengthMap.put("chr18", 76117153);
		Hg18ChrLengthMap.put("chr19", 63811651);
		Hg18ChrLengthMap.put("chr20", 62435964);
		Hg18ChrLengthMap.put("chr21", 46944323);
		Hg18ChrLengthMap.put("chr22", 49691432);
		Hg18ChrLengthMap.put("chrX", 154913754);
		Hg18ChrLengthMap.put("chrY", 57772954);
	}

	/**
	 * convert Yeast Roman Express name to common used num express
	 * 
	 * @param romanChr
	 *            Roman format of yeast chr name
	 * @return numerical format of yeast chr name
	 */
	public static String convertYeastRomanChr2Num(String romanChr) {
		int index = YeastChrNameList_Roman.indexOf(romanChr);

		if (index >= 0) {
			return YeastChrNameList.get(index);
		}

		return romanChr;
	}

	/**
	 * convert Yeast Numerical Express name to Roman expression
	 * 
	 * @param numChr
	 *            numerical format of yeast chr name
	 * @return romanChr Roman format of yeast chr name
	 */
	public static String convertYeastNumericalChr2Roman(String numChr) {
		int index = YeastChrNameList.indexOf(numChr);

		if (index >= 0) {
			return YeastChrNameList_Roman.get(index);
		}

		return numChr;
	}

}
