package com.zhengyiyu.ffn;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;

import com.zhengyiyu.common.FolderPath;
import com.zhengyiyu.feature.FeatureManager;
import com.zhengyiyu.feature.SequenceFeature;
import com.zhengyiyu.handler.NucleosomeDataHandler;
import com.zhengyiyu.handler.SequenceDataHandler;
import com.zhengyiyu.handler.SequenceReader;
import com.zhengyiyu.handler.SequenceReader_Yeast;
import com.zhengyiyu.handler.SequenceReader_hg18;
import com.zhengyiyu.obj.Gene;
import com.zhengyiyu.obj.Nucleosome;
import com.zhengyiyu.util.StringUtil;

/**
 * @author Yiyu
 * Generate the input profile for FFN algorithm
 */
public class Step1_Profile_Generation {
	public static void main(String[] args) throws IOException {
		// Can have two parameters, the species and cell status, otherwise the default will be yeast
		
		String speciesName = "yeast";
		String cellStatus = "";
		
		if (args.length == 2) {
			speciesName = args[0];
			if (!speciesName.equals("yeast")) {
				cellStatus = args[1];
			} else {
				cellStatus = "";
			}
		}
		
		int[] broaderRegion = {-2000, 2000};
		int[] promoterRegion = {-1000, 0};
		
		int minLinkerLength = 50;
		int normalLength = 147;

		System.out.println("###Species:" + speciesName + (cellStatus.equals("") ? "" : ", cell status:" + cellStatus));
		
		String rootFolder = FolderPath.OUTPUT_FOLDER_Full + "pattern_" + speciesName + cellStatus + File.separator;
		
		boolean allowNucleosomeInMargin = true;
		
		String caseName = "nucleosome";
		String controlName = "linker";
		
		String nucleosomeProfileName = rootFolder + speciesName + cellStatus + "_" + caseName + "_profile";
		String linkerProfileName = rootFolder + speciesName + cellStatus + "_" + controlName + "_profile";
		
		BufferedWriter nucleosomeList = new BufferedWriter(new FileWriter(rootFolder + speciesName + cellStatus + "_" + caseName + "_list"));
		BufferedWriter linkerList = new BufferedWriter(new FileWriter(rootFolder + speciesName + cellStatus + "_" + controlName + "_list"));
		
		BufferedWriter nucleosomeProfile = new BufferedWriter(new FileWriter(nucleosomeProfileName));
		BufferedWriter linkerProfile = new BufferedWriter(new FileWriter(linkerProfileName));
		
		int count = 0;
		
		nucleosomeProfile.write("### include all sequence features with 4 level on all nucleosome");
		nucleosomeProfile.newLine();
		
		linkerProfile.write("### include all sequence features with 4 level on all linkers length >= 50 bp");
		linkerProfile.newLine();
		
		// only use the nucleosome and linkers in range to find the pattern
		ArrayList<SequenceFeature> attrs = FeatureManager.restoreDiscretizedFeatures(speciesName, cellStatus, rootFolder);
		System.out.println(attrs.size());
		
//		LogUtil log = new LogUtil(speciesName + cellStatus);
//		log.redirectSysout();
		
		ArrayList<Gene> genesWithTranscript = NucleosomeDataHandler.restoreGeneTranscriptNucleosome(rootFolder, speciesName, cellStatus, allowNucleosomeInMargin, promoterRegion, broaderRegion);
		
		// prepare data
		SequenceReader sequenceReader = speciesName.equals("yeast") ? SequenceReader_Yeast.getSGD200602Reader() : SequenceReader_hg18.getReader();
		
		// make sure it is none repeated!
		HashSet<String> nucleosomeSet = new HashSet<String>();
		
		for (Gene g : genesWithTranscript) {
			@SuppressWarnings("unchecked")
			ArrayList<Nucleosome> nsandlinkers = (ArrayList<Nucleosome>) g.getInfo().get("nsandlinkers");
			
			int contentStartPosition = (Integer) g.getInfo().get("seqstart");
			int contentEndPosition = (Integer) g.getInfo().get("seqend");
			String geneSequence = (String) g.getInfo().get("seq");
			
			for (Nucleosome ns : nsandlinkers) {
				if (nucleosomeSet.contains(ns.toCompactString())) {
					// already existed in another promoter region
					continue;
				} else {
					nucleosomeSet.add(ns.toCompactString());
					
					// only filter is the length of linker
					if (ns.isLinker()) {
						if (ns.length() < minLinkerLength) {
							continue;
						}
					}
					
					// for delocalize, it is changed
					
					int dyad = ns.dyad();
					
					int extendedLength = ns.length();
					
					if (ns.getStatus() == 2) {
						extendedLength = normalLength + 25 * 2; // extend to eachside 25 bp
					} else if (ns.isLinker()) {
						if (ns.length() > normalLength) {
							extendedLength = ns.length();
						} else {
							extendedLength =  normalLength + 25 * 2;
						}
					} else if (ns.getStatus() == 1) {
						if (ns.length() <= normalLength + 25 * 2) {
							// <= 197 bp, we do not extend it
							extendedLength = ns.length();
						} else {
							// we use the highest score position and extend to 197 bp
							dyad = ns.getMiddlePosition();
							extendedLength =  normalLength + 25 * 2;
						}
					}
					
					// we do not care about the strand! as long as the nucleosome is in the gene region, it is ok!
					// for each nucleosome it should be extened to the nucleosomelength
					
					int nucleosomeStartPosition = dyad - extendedLength / 2;
					int nucleosomeEndPosition = dyad + extendedLength / 2;
					
					String sequence = SequenceDataHandler.getExtendedNucleosomeSequenceFromGene(contentStartPosition, contentEndPosition, geneSequence, ns.getChromosome(), nucleosomeStartPosition, nucleosomeEndPosition, 
							sequenceReader, normalLength);
					
					// only filter N
					if (StringUtil.containsRepeatMaskN(sequence)) {
						continue;
					}
					
					String s = "###\t" + ns.toString() + "\t" + extendedLength + "\t" + dyad + "\t" + nucleosomeStartPosition + "\t" + nucleosomeEndPosition;
					
					if (ns.getStatus() == 2 || ns.getStatus() == 1) {
						nucleosomeProfile.write(s);
						nucleosomeProfile.newLine();
						nucleosomeList.write(s);
						nucleosomeList.newLine();
					} else {
						// linker
						linkerProfile.write(s);
						linkerProfile.newLine();
						linkerList.write(s);
						linkerList.newLine();
					}
					
					// calculate values
					
					sequence = sequence.toUpperCase();
					
					for (int i = 0; i < sequence.length() - normalLength; i++) {
						String valueLine = "";
						
						String content = sequence.substring(i, i + normalLength);
						
						int[] data = new int[attrs.size()];
						
						int index = 0;
						for (; index < attrs.size(); index++) {
							SequenceFeature sa = attrs.get(index);
							double value = sa.calculateFeatureValue(content);
							int subIndex = sa.getPatternIndex(value);
							valueLine += subIndex + "\t";
							data[index] = subIndex;
						}
						
						if (ns.getStatus() == 2 || ns.getStatus() == 1) {
							nucleosomeProfile.write(valueLine);
							nucleosomeProfile.newLine();
						} else {
							// linker
							linkerProfile.write(valueLine);
							linkerProfile.newLine();
						}
						
						count++;
						
						if (count % 500000 == 0) {
							System.out.println(count);
							nucleosomeProfile.flush();
							linkerProfile.flush();
						}
					}
					
				}
			}
		}
		
		
		// also need to generate a linker value list...
		// but as linker do not have such kind of linker length feature... so do not, and we cannot calculate zscore for linker
		nucleosomeProfile.flush();
		linkerProfile.flush();
		
		nucleosomeProfile.close();
		linkerProfile.close();
		
		nucleosomeList.close();
		linkerList.close();
		
		System.out.println("Finished");
	}
}
