package com.zhengyiyu.ffn;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import com.zhengyiyu.common.FolderPath;
import com.zhengyiyu.feature.FeatureManager;
import com.zhengyiyu.feature.SequenceFeature;
import com.zhengyiyu.handler.PatternHandler;
import com.zhengyiyu.obj.Pattern;
import com.zhengyiyu.util.FileUtil;

public class Step3_PatternSupportCount {
	public static void main(String[] args) throws IOException {
		
		String speciesName = "yeast";
		String cellStatus = "";
		
		String caseName = "nucleosome";
		String controlName = "linker";
		double alpha = 0.3;
		
		if (args.length >= 5) {
			speciesName = args[0];
			if (!speciesName.equals("yeast")) {
				cellStatus = args[1];
			} else {
				cellStatus = "";
			}
			
			caseName = args[2];
			controlName = args[3];
			
			alpha = Double.parseDouble(args[4]);
		}
		
		// parameters
		
		System.out.println("###Species:" + speciesName + (cellStatus.equals("") ? "" : ", cell status:" + cellStatus));
		
		String rootFolder = FolderPath.OUTPUT_FOLDER_Full + "pattern_" + speciesName + cellStatus + File.separator;

		// int profileSize = PatternHandler.getProfileSize(speciesName, cellStatus, caseName);
		
		ArrayList<SequenceFeature> discretizedFeatures = FeatureManager.restoreDiscretizedFeatures(speciesName, cellStatus, rootFolder);
		
		// restore patterns with count
		// also need to remove the linkerlength containing patterns
		
		// first step count!
		int featureSize = discretizedFeatures.size();
		
		String caseProfileName = rootFolder + speciesName + cellStatus + "_" + caseName + "_profile";
		String controlProfileName = rootFolder + speciesName + cellStatus + "_" + controlName + "_profile";
		
		ArrayList<int[]> caseData = PatternHandler.restoreProfileData(caseProfileName, null, featureSize);
		ArrayList<int[]> controlData = PatternHandler.restoreProfileData(controlProfileName, null, featureSize);
		
		int profileSize = caseData.size();
		int beta = (int) (profileSize * alpha);
		System.out.println("###beta:" + beta);
		System.out.println("###caseData:" + caseData.size());
		System.out.println("###controlData:" + controlData.size());
		
		if (profileSize != caseData.size()) {
			System.out.println("!!!!impossible that profile size is not right");
		}
		// to be quick, we start from length2 patterns
		String countFolder = rootFolder + "count" + alpha + File.separator;
		FileUtil.createFolder(countFolder);
		int length = 2;
		
		String countFile = countFolder + caseName + "_" + controlName + "_length" + length + "_count";
		
		ArrayList<Pattern> filteredLength2Patterns = null;
		
		if (new File(countFile).exists()) {
			filteredLength2Patterns = PatternHandler.restorePatternCountFile(countFile);
		} else {
			filteredLength2Patterns =  PatternHandler.countLength2Patterns(beta, discretizedFeatures, caseData, caseName + "_count");
			System.out.println("###Length" + length + " pattern size after filter (beta=" + beta + "):" + filteredLength2Patterns.size());
			Collections.sort(filteredLength2Patterns, PatternHandler.getCountComparator());
			
			PatternHandler.countPatternNumber(filteredLength2Patterns, -1, controlData, controlName + "_count");
			PatternHandler.recordFilterLengthPatterns(countFile, filteredLength2Patterns, discretizedFeatures, caseName, controlName);
		}
		
		System.out.println("###Length" + length + " pattern size after filter (beta=" + beta + "):" + filteredLength2Patterns.size());
		ArrayList<ArrayList<Pattern>> patternsOfDifferentLength = new ArrayList<ArrayList<Pattern>>();
		
		while (true) {
			ArrayList<Pattern> nextLengthPatterns = null;
			ArrayList<Pattern> nextLengthPatternsFiltered = null;
			length++;
			
			countFile = countFolder + caseName + "_" + controlName + "_length" + length + "_count";
			
			if (new File(countFile).exists()) {
				nextLengthPatternsFiltered = PatternHandler.restorePatternCountFile(countFile);
			} else {
				if (patternsOfDifferentLength.size() == 0) {
					nextLengthPatterns =  PatternHandler.combineDimers(filteredLength2Patterns, discretizedFeatures);
				} else {
					nextLengthPatterns = PatternHandler.combineDimers(patternsOfDifferentLength.get(patternsOfDifferentLength.size() - 1), discretizedFeatures);
				}
				
				if (nextLengthPatterns.size() == 0) {
					break;
				}
				
				System.out.println("###Length" + length + " pattern size:" + nextLengthPatterns.size());
				nextLengthPatternsFiltered = PatternHandler.countPatternNumber(nextLengthPatterns, beta, caseData,  caseName + "_count");
				System.out.println("###Length" + length + " pattern size after filters:" + nextLengthPatternsFiltered.size());
				
				Collections.sort(nextLengthPatternsFiltered, PatternHandler.getCountComparator());

				PatternHandler.countPatternNumber(nextLengthPatternsFiltered, -1, controlData, controlName + "_count");
				PatternHandler.recordFilterLengthPatterns(countFile, nextLengthPatternsFiltered, discretizedFeatures, caseName, controlName);
			}
			
			// need to record the result into file
			
			patternsOfDifferentLength.add(nextLengthPatternsFiltered);
			
			// control count here
		}
		
		System.out.println("Finished");
	}
}
