package com.zhengyiyu.handler;

import java.util.ArrayList;
import java.util.HashMap;

import com.zhengyiyu.obj.Nucleosome;
import com.zhengyiyu.obj.Sequence;
import com.zhengyiyu.util.StringUtil;

/**
 * @author Yiyu Zheng
 *
 */
public class SequenceDataHandler {

	public static <T extends Sequence> HashMap<String, ArrayList<T>> groupSequenceByChromosome(ArrayList<T> sequences) {
			
		HashMap<String, ArrayList<T>> nucleosomeChrMap = new HashMap<String, ArrayList<T>>();
		
		for (T ns : sequences) {
			if (nucleosomeChrMap.containsKey(ns.getChromosome())) {
				nucleosomeChrMap.get(ns.getChromosome()).add(ns);
			} else {
				ArrayList<T> chrNucleosomeList = new ArrayList<T>();
				chrNucleosomeList.add(ns);
				nucleosomeChrMap.put(ns.getChromosome(), chrNucleosomeList);
			}
		}
		return nucleosomeChrMap;
	}

	public static String getSequenceFromGene(int contentStartPosition, int contentEndPosition, String geneSequence, Nucleosome ns, SequenceReader sequenceReader, int nucleosomeLength) {
		
		// we do not care about the strand! as long as the nucleosome is in the gene region, it is ok!
		// for each nucleosome it should be extened to the nucleosomelength
		
		int dyad = ns.getStatus() == 1 ? ns.getMiddlePosition() : ns.dyad();
		
		int nucleosomeStartPosition = dyad - nucleosomeLength / 2;
		int nucleosomeEndPosition = dyad + nucleosomeLength / 2;
		
		if (nucleosomeStartPosition >= contentStartPosition && nucleosomeEndPosition <= contentEndPosition) {
			return StringUtil.subString(geneSequence, nucleosomeStartPosition - contentStartPosition, nucleosomeEndPosition - contentStartPosition + 1);
		} else {
			return sequenceReader.getSequenceByLength(ns.getChromosome(), nucleosomeStartPosition, nucleosomeLength);
		}
	}
	
	public static String getExtendedNucleosomeSequenceFromGene(int contentStartPosition, int contentEndPosition, String geneSequence, String chr, int nucleosomeStartPosition, int nucleosomeEndPosition, SequenceReader sequenceReader, int nucleosomeLength) {
		
		if (nucleosomeStartPosition >= contentStartPosition && nucleosomeEndPosition <= contentEndPosition) {
			return StringUtil.subString(geneSequence, nucleosomeStartPosition - contentStartPosition, nucleosomeEndPosition - contentStartPosition + 1);
		} else {
			return sequenceReader.getSequenceByLength(chr, nucleosomeStartPosition, nucleosomeLength);
		}
	}

}
