package com.zhengyiyu.obj;

/**
 * @author yiyu
 * also represent linker sequence
 */
public class Nucleosome extends Sequence {
	
	private int status; //  0: linker DNA, 2: well-positioned nucleosome, 1: delocalized nucleosome,
	
	private int originalLength; // because sometimes we extend nucleosome to a certain length
	
	private boolean containRepeatMask; // whether this nucleosome contains repeatMask String "N"
	
	private boolean containTandemRepeat; // whether this nucleosome contains tanden repeat, String "acgt";
	
	private boolean containPatterns; // whether this nucleosome contains certain patterns
	
	private int positionIndex; // the relative position to TSS, -1 : first upstream, 0 : overlap with TSS (if most part of the nucleosome are upstream!!!), 1 downstream of TSS
	
	private int middlePosition; // the highest score position of delocalized nucleosome

	public Nucleosome(Sequence seq, int status) {
		super(seq);
		this.originalLength = this.length();
		this.status = status;
	}
	
	public Nucleosome(String chromosome, int startPosition, int endPosition, int status) {
		super(chromosome, startPosition, endPosition);
		this.originalLength = this.length();
		this.status = status;
	}

	/**
	 * @return the status
	 */
	public int getStatus() {
		return status;
	}

	/**
	 * @param status the status to set
	 */
	public void setStatus(int status) {
		this.status = status;
	}
	
	/**
	 * @return the originalLength
	 */
	public int getOriginalLength() {
		return originalLength;
	}

	/**
	 * @param originalLength the originalLength to set
	 */
	public void setOriginalLength(int originalLength) {
		this.originalLength = originalLength;
	}

	/**
	 * @return the containRepeatMask
	 */
	public boolean isContainRepeatMask() {
		return containRepeatMask;
	}

	/**
	 * @param containRepeatMask the containRepeatMask to set
	 */
	public void setContainRepeatMask(boolean containRepeatMask) {
		this.containRepeatMask = containRepeatMask;
	}

	/**
	 * @return the containTandemRepeat
	 */
	public boolean isContainTandemRepeat() {
		return containTandemRepeat;
	}

	/**
	 * @param containTandemRepeat the containTandemRepeat to set
	 */
	public void setContainTandemRepeat(boolean containTandemRepeat) {
		this.containTandemRepeat = containTandemRepeat;
	}

	/**
	 * @return the containPatterns
	 */
	public boolean isContainPatterns() {
		return containPatterns;
	}

	/**
	 * @param containPatterns the containPatterns to set
	 */
	public void setContainPatterns(boolean containPatterns) {
		this.containPatterns = containPatterns;
	}

	/**
	 * @return the positionIndex
	 */
	public int getPositionIndex() {
		return positionIndex;
	}

	/**
	 * @param positionIndex the positionIndex to set
	 */
	public void setPositionIndex(int positionIndex) {
		this.positionIndex = positionIndex;
	}

	/**
	 * @return the middlePosition
	 */
	public int getMiddlePosition() {
		return middlePosition;
	}

	/**
	 * @param middlePosition the middlePosition to set
	 */
	public void setMiddlePosition(int middlePosition) {
		this.middlePosition = middlePosition;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Nucleosome) {
			this.toCompactString().equals(((Nucleosome) obj).toCompactString());
		} 
		
		return false;
	}
	
	@Override
	public int hashCode() {
		return this.toCompactString().hashCode();
	}
	
	@Override
	public String toString() {
		return super.toString() + "\t" + this.dyad() + "\t" + this.length() + "\t" + this.status;
	}

	/**
	 * @return
	 * whether this is a linker, the status = 3 fuzzy one is counted as linker too
	 */
	public boolean isLinker() {
		return this.status == 0 || this.status == 3;
	}
}
