package com.zhengyiyu.obj;

import java.util.ArrayList;
import java.util.HashMap;

public class Pattern implements Comparable<Pattern> {
	// e.g., A1C3, A1C3F2
	private ArrayList<String> sub; // the label of feature
	private ArrayList<Integer> subFeatures; // the index of feature
	private ArrayList<Integer> subIndexs; // the subcategory of feature
	
	private HashMap<String, Object> info;
	
	private int count;
	
	private double patternScore;
	
	private int rank;
	
	private String patternDetail;
	
	private double normalizedScore;
	
	private double normalizedRank;
	
	// feature, and value
	private HashMap<Integer, ArrayList<Integer>> featureIndexes;
	
	public Pattern(ArrayList<String> sub) {
		super();
		this.sub = sub;
		
		this.compileSingleFeature();
		
		this.info = new HashMap<String, Object>();
	}

	public Pattern(String pattern, int subLength) {
		super();
		this.sub = new ArrayList<String>();
		for (int i = 0; i < pattern.length(); i = i + subLength) {
			sub.add(pattern.substring(i, i + subLength));
		}
		
		this.compileSingleFeature();
		
		this.info = new HashMap<String, Object>();
		
	}
	
	public Pattern(String pattern, int subLength, String patternDetail) {
		this(pattern, subLength);
		this.setPatternDetail(patternDetail);
	}

	/**
	 * @return the sub
	 */
	public ArrayList<String> getSub() {
		return sub;
	}

	/**
	 * @param sub the sub to set
	 */
	public void setSub(ArrayList<String> sub) {
		this.sub = sub;
	}
	
	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		for (String s : sub) {
			if (s.length() > 2) {
				sb.append("[" + s + "]");
			} else {
				sb.append(s);
			}
		}
		return sb.toString();
	}
	
	/**
	 * @return the count
	 */
	public int getCount() {
		return count;
	}

	/**
	 * @param count the count to set
	 */
	public void setCount(int count) {
		this.count = count;
	}

	public String toPatternNameString() {
		StringBuffer sb = new StringBuffer();
		for (String s : sub) {
			sb.append(s.substring(0, 1));
		}
		return sb.toString();
	}
	
	public void compileSingleFeature() {
		subFeatures = new ArrayList<Integer>();
		subIndexs = new ArrayList<Integer>();
		
		for (String s : sub) {
			int index = (int)(s.substring(0, 1).charAt(0));
			if (index < 91) {
				subFeatures.add(index - 65);
			} else {
				subFeatures.add(index - 71);
			}
			subIndexs.add(Integer.parseInt(s.substring(1, 2)));
		}
	}
	
	public ArrayList<Integer> getSubFeatures() {
		return subFeatures;
	}

	public void setSubFeatures(ArrayList<Integer> features) {
		this.subFeatures = features;
	}

	public ArrayList<Integer> getSubIndexs() {
		return subIndexs;
	}

	public void setSubIndexs(ArrayList<Integer> subIndexs) {
		this.subIndexs = subIndexs;
	}

	public HashMap<String, Object> getInfo() {
		return info;
	}

	public void setInfo(HashMap<String, Object> info) {
		this.info = info;
	}

	public HashMap<Integer, ArrayList<Integer>> getFeatureIndexes() {
		return featureIndexes;
	}

	public void setFeatureIndexes(
			HashMap<Integer, ArrayList<Integer>> featureIndexes) {
		this.featureIndexes = featureIndexes;
	}

	public double getPatternScore() {
		return patternScore;
	}

	public void setPatternScore(double score) {
		this.patternScore = score;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Pattern) {
			return obj.toString().equals(this.toString());
		} else {
			return false;
		}
	}
	
	@Override
	public int hashCode() {
		return this.toString().hashCode();
	}

	@Override
	public int compareTo(Pattern o) {
		
		int size = Math.min(this.sub.size(), o.sub.size());
		for (int i = 0; i < size; i++) {
			if (this.sub.get(i).compareTo(o.sub.get(i)) != 0) {
				return this.sub.get(i).compareTo(o.sub.get(i));
			}
		}
		return this.sub.size() - o.sub.size();
	}

	public String getPatternDetail() {
		return patternDetail;
	}

	public void setPatternDetail(String patternDetail) {
		this.patternDetail = patternDetail;
	}

	public int getRank() {
		return rank;
	}

	public void setRank(int rank) {
		this.rank = rank;
	}
	
	/**
	 * @return the normalizedScore
	 */
	public double getNormalizedScore() {
		return normalizedScore;
	}

	/**
	 * @param normalizedScore the normalizedScore to set
	 */
	public void setNormalizedScore(double normalizedScore) {
		this.normalizedScore = normalizedScore;
	}

	/**
	 * @return the normalizedRank
	 */
	public double getNormalizedRank() {
		return normalizedRank;
	}

	/**
	 * @param normalizedRank the normalizedRank to set
	 */
	public void setNormalizedRank(double normalizedRank) {
		this.normalizedRank = normalizedRank;
	}
	
}
