package com.zhengyiyu.obj;

import java.text.DecimalFormat;
import java.util.HashMap;

import Jama.EigenvalueDecomposition;
import Jama.Matrix;

import com.zhengyiyu.util.ArrayUtil;
import com.zhengyiyu.util.DoubleUtil;

public class TransitionMatrix {
	public static double[][] Matrix_Power0 = { { 1, 0, 0, 0 }, { 0, 1, 0, 0 }, { 0, 0, 1, 0 }, { 0, 0, 0, 1 } };

	private String label;
	private String name;
	private double[][] matrix;
	private double[][] countMatrix;

	private double[] pi; // suppose this is the eigenvector

	private int maxPower = 0;

	private HashMap<Integer, double[][]> poweredMatrixMap;

	public TransitionMatrix(String label, String name) {
		super();
		this.label = label;
		this.name = name;

		// powerValue = new HashMap<String, HashMap<Integer,Double>>();

		poweredMatrixMap = new HashMap<Integer, double[][]>();
		poweredMatrixMap.put(0, Matrix_Power0);
		this.maxPower = 0;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public double[][] getMatrix() {
		return matrix;
	}

	public void setMatrix(double[][] matrix) {
		this.matrix = matrix;
	}

	public double[] getPi() {
		return pi;
	}

	public void setPi(double[] pi) {
		this.pi = pi;
	}

	public void transformMatrix() {
		Matrix t = new Matrix(this.getMatrix());
		Matrix transposedMatrix = t.transpose();

		ArrayUtil.printDoubleArray("Markov chain modeled Transition Matrix:", this.getMatrix());
		
		for (double[] dd : this.getMatrix()) {
			System.out.print("(");
			for (double d : dd) {
				System.out.print(d + ", ");
			}
			
			System.out.print(("), "));
		}
		
		System.out.println("");
		

		ArrayUtil.printDoubleArray("after transpose:", transposedMatrix.getArray());

		EigenvalueDecomposition ed = new EigenvalueDecomposition(transposedMatrix);
		double[] d = ed.getRealEigenvalues();
		
		ArrayUtil.printArray("EigenValues", d);
		ArrayUtil.printDoubleArray("Vector", ed.getV().getArray());
		
		Matrix m = ed.getV();

		for (int i = 0; i < d.length; i++) {
			if (Math.abs(d[i] - 1.0d) < 0.000001) {
				// this is the index column of pi
				double[] pi = new double[d.length];

				double sum = 0.0;
				for (int row = 0; row < pi.length; row++) {
					pi[row] = m.getArray()[row][i];
					sum += m.getArray()[row][i];
				}
				
				ArrayUtil.printArray("Pi Before Normalize to 1", pi);
				
				// then we need to normalize it to sum=1, value between[0, 1],
				// as it seems that all the values are
				if (sum != 0.0) {
					for (int row = 0; row < pi.length; row++) {
						pi[row] = pi[row] / sum;
					}
				}

				this.setPi(pi);
				break;
			}
		}

		
		ArrayUtil.printArray("Pi", this.getPi());
		
		System.out.println("");
	}

	@Override
	public String toString() {
		DecimalFormat df3 = DoubleUtil.df3;

		StringBuffer sb = new StringBuffer();
		sb.append(this.getLabel() + "\t" + this.getName() + "\r\n");

		for (double[] darray : this.getMatrix()) {
			for (double d : darray) {
				sb.append("\t" + df3.format(d));
			}
			sb.append("\r\n");
		}

		for (double d : this.getPi()) {
			sb.append("\t" + df3.format(d));
		}
		sb.append("\r\n");
		return sb.toString();
	}

	public double[][] getCountMatrix() {
		return countMatrix;
	}

	public void setCountMatrix(double[][] countMatrix) {
		this.countMatrix = countMatrix;
	}

	public HashMap<Integer, double[][]> getPoweredMatrixMap() {
		return poweredMatrixMap;
	}

	public void setPoweredMatrixMap(
			HashMap<Integer, double[][]> poweredMatrixMap) {
		this.poweredMatrixMap = poweredMatrixMap;
	}

	public int getMaxPower() {
		return maxPower;
	}

	public void setMaxPower(int maxPower) {
		this.maxPower = maxPower;
	}

	public double[][] getPoweredMatrix(int power) {
		if (power < 0) {
			System.out.println("Matrix Power cannot be than zero, current power:" + power);
			return null;
		}

		if (power <= maxPower) {
			return poweredMatrixMap.get(power);
		} else {
			if (power == 1) {
				poweredMatrixMap.put(1, this.matrix);
				maxPower = 1;
				return this.matrix;
			}

			double[][] maxPowerArray = poweredMatrixMap.get(maxPower);
			Matrix singleMatrix = new Matrix(matrix);

			Matrix poweredMatrix = new Matrix(maxPowerArray);

			int i = maxPower + 1;

			for (; i <= power; i++) {
				// we need to change it to a JAMA matrix

				poweredMatrix = poweredMatrix.times(singleMatrix);
				poweredMatrixMap.put(i, poweredMatrix.getArrayCopy());
				maxPower = i;
			}

//			 ArrayUtil.printDoubleArray(this.getLabel() + " " + power, poweredMatrix.getArrayCopy());

			return poweredMatrix.getArrayCopy();
		}
	}

}
